<?php
class PAP_WC_POINTS_REWARDS {

	public $settings = array();
	public $count_referrals_processed;
	public $pending_admin_notice;

	public function __construct($settings) {
		$this->settings = $settings;
		$this->setup();
		
	}
	
	public function setup() {
		
		// Set the description for points log
		add_filter( 'wc_points_rewards_event_description', array( $this, 'add_points_log_description' ), 10, 3 );
		
		//$this->conversion_ratio = get_option('wc_points_rewards_redeem_points_ratio');
		$override_enable = affiliate_wp()->settings->get('pap_wcpr_convert_ratio_override_enable');
		$override_ratio = affiliate_wp()->settings->get('pap_wcpr_convert_ratio_override_ratio');
		
		//if( isset($this->settings['pap_wcpr_convert_ratio_override_enable']) && isset($this->settings['pap_wcpr_convert_ratio_override_ratio']) ) {
		
			// If no conversion overrides is enabled, use the ones set in the points and rewards settings
			if( ($override_enable=='1') && !empty($override_ratio) ) {
				$this->conversion_ratio = $override_ratio;
			} else {
				$this->conversion_ratio = get_option('wc_points_rewards_redeem_points_ratio');
			}
		
		//}

		$this->conversion_rounding = get_option('wc_points_rewards_earn_points_rounding');
		
	}
	
	// Add admin settings if required.
	public static function admin_settings($settings) {
		
		$settings['pap']['pap_wcpr_convert_ratio_override_enable'] = array(
			'name' => __( 'Override the conversion ratio', 'affiliate-wp-pay-as-points' ),
			'desc' => __( 'Check this box to use a different redemption conversion rate to that which in the Point & rewards settings. Woocommerce > Settings > Points & Rewards', 'affiliate-wp-pay-as-points' ),
			'type' => 'checkbox'
		);
						
		$settings['pap']['pap_wcpr_convert_ratio_override_ratio'] = array(
			'name' => '<br>'.__( 'Conversion rate override', 'affiliate-wp-pay-as-points' ),
			'desc' => '<br>'.__( 'If enabled above, enter the redemption rate to use.</br>Format points:dollars. Example 100:1 equals 100 points for every referral dollar.', 'affiliate-wp-pay-as-points' ),
			'type' => 'text'
		);
		
		return $settings;
		
	}
	
	// Calculate the points to allocate. Required Method
	public function calculate_points($referral_amount) {

		$conversion_ratio_parts = explode(":", $this->conversion_ratio);
		$conversion_ratio_multiplier = ($conversion_ratio_parts[0]/$conversion_ratio_parts[1]);
		$points = ($referral_amount*$conversion_ratio_multiplier);
		
		if($this->conversion_rounding == 'round') $points = round($points);
		if($this->conversion_rounding == 'floor') $points = floor($points);
		if($this->conversion_rounding == 'ceil') $points = ceil($points);
		
		return $points;
		
	}
	
	// Log the points. Required Method
	public function process_points($referral_id, $points, $user_id, $debit_credit) {
		
		if(!isset($this->count_referrals_processed)) $this->count_referrals_processed = 0;
		
		if ( ! empty( $points ) ) {
				 
			if($debit_credit=='credit')  {
				if(WC_Points_Rewards_Manager::increase_points($user_id, $points,'converted-affiliate-commission')) {
				
					$this->count_referrals_processed ++;
					if(!isset($this->pending_admin_notice)) $this->pending_admin_notice = 'successfully converted to loyalty points!';
	
				}
				
			} elseif($debit_credit=='debit') {
				if(WC_Points_Rewards_Manager::decrease_points($user_id, $points,'reverted-affiliate-commission')) {
				
					$this->count_referrals_processed ++;
					if(!isset($this->pending_admin_notice)) $this->pending_admin_notice = ' reverted fom loyalty points!';
					
				}
			}

		}
		
	}
	
	// Set the points log description
	public function add_points_log_description( $event_description, $event_type='', $event='') {	
	
		switch ($event_type) {
		case 'converted-affiliate-commission': 
		$event_description = 'Converted affiliate commission';
		break;
		case 'reverted-affiliate-commission': 
		$event_description = 'Reverted affiliate commission';
		break;
		}

		return $event_description;
		
	}
	
} // End of class

?>